<?php

  // Database connection
  $db   ="";
  $host ="localhost";
  $user ="";
  $pswd ="";

  // Name of reference
  $reference="reference";

  // External link from track
  $external_link="";

  // Initial name in front of chromosome name
  // usage: If $chromosome_name="chr";
  //        "01" -> "chr01"
  //        "02" -> "chr02"
  //        ..
  //        "12" -> "chr12"
  $chromosome_name="";

  // Maximum level of zoom for viewing each nucleotide
  $nuc_max="4k";
  
  // Maximum of depth value in display
  $depth_max=140;

  // Maximum of SNP count by each level of zoom
  $snp_100k=1000;
  $snp_90k=900;
  $snp_80k=800;
  $snp_70k=700;
  $snp_60k=600;
  $snp_50k=500;
  $snp_40k=400;
  $snp_30k=300;
  $snp_20k=200;
  $snp_10k=100;
  $snp_9k=90;
  $snp_8k=80;
  $snp_7k=70;
  $snp_6k=60;
  $snp_5k=50;
  $snp_4k=40;
  $snp_3k=30;
  $snp_2k=20;
  $snp_1k=10;
  $snp_900b=9;
  $snp_800b=8;
  $snp_700b=7;
  $snp_600b=6;
  $snp_500b=5;
  $snp_400b=4;
  $snp_300b=3;
  $snp_200b=2;
  $snp_100b=1;
  $snp_90b=1;
  $snp_80b=1;
  $snp_70b=1;
  $snp_60b=1;
  $snp_50b=1;
  $snp_40b=1;
  $snp_30b=1;
  $snp_20b=1;
  $snp_10b=1;
  $snp_9b=1;
  $snp_8b=1;
  $snp_7b=1;
  $snp_6b=1;
  $snp_5b=1;
  $snp_4b=1;
  $snp_3b=1;
  $snp_2b=1;
  $snp_1b=1;
  
  
  
  // Maximum of INDEL count by each level of zoom
  $indel_100k=100;
  $indel_90k=90;
  $indel_80k=80;
  $indel_70k=70;
  $indel_60k=60;
  $indel_50k=50;
  $indel_40k=40;
  $indel_30k=30;
  $indel_20k=20;
  $indel_10k=10;
  $indel_9k=6;
  $indel_8k=6;
  $indel_7k=6;
  $indel_6k=6;
  $indel_5k=6;
  $indel_4k=3;
  $indel_3k=3;
  $indel_2k=3;
  $indel_1k=3;
  $indel_900b=1;
  $indel_800b=1;
  $indel_700b=1;
  $indel_600b=1;
  $indel_500b=1;
  $indel_400b=1;
  $indel_300b=1;
  $indel_200b=1;
  $indel_100b=1;
  $indel_90b=1;
  $indel_80b=1;
  $indel_70b=1;
  $indel_60b=1;
  $indel_50b=1;
  $indel_40b=1;
  $indel_30b=1;
  $indel_20b=1;
  $indel_10b=1;
  $indel_9b=1;
  $indel_8b=1;
  $indel_7b=1;
  $indel_6b=1;
  $indel_5b=1;
  $indel_4b=1;
  $indel_3b=1;
  $indel_2b=1;
  $indel_1b=1;

?>